@@ -23,12 +23,8 @@
 #include <config.h>
 #include <compat.h>
 
-#include <sys/stat.h>
-
 #include <errno.h>
-#include <inttypes.h>
 #include <string.h>
-#include <fcntl.h>
 #include <unistd.h>
 #include <stdbool.h>
 #include <stdlib.h>
@@ -49,6 +45,7 @@
 #include <dpkg/varbuf.h>
 #include <dpkg/fdio.h>
 #include <dpkg/buffer.h>
+#include <dpkg/meminfo.h>
 #include <dpkg/command.h>
 #include <dpkg/compress.h>
 #if USE_LIBZ_IMPL == USE_LIBZ_IMPL_NONE || \
@@ -56,12 +53,9 @@
     !defined(WITH_LIBBZ2)
 #include <dpkg/subproc.h>
 
-static void DPKG_ATTR_SENTINEL
-fd_fd_filter(int fd_in, int fd_out, const char *desc, const char *delenv[],
-             const char *file, ...)
+static void
+fd_fd_filter(struct command *cmd, int fd_in, int fd_out, const char *delenv[])
 {
-	va_list args;
-	struct command cmd;
 	pid_t pid;
 	int i;
 
@@ -79,15 +73,30 @@ fd_fd_filter(int fd_in, int fd_out, const char *desc, const char *delenv[],
 		for (i = 0; delenv[i]; i++)
 			unsetenv(delenv[i]);
 
-		command_init(&cmd, file, desc);
-		command_add_arg(&cmd, file);
-		va_start(args, file);
-		command_add_argv(&cmd, args);
-		va_end(args);
-
-		command_exec(&cmd);
+		command_exec(cmd);
 	}
-	subproc_reap(pid, desc, 0);
+	subproc_reap(pid, cmd->name, 0);
+}
+
+static void
+command_compress_init(struct command *cmd, const char *name, const char *desc,
+                      int level)
+{
+	static char combuf[6];
+
+	command_init(cmd, name, desc);
+	command_add_arg(cmd, name);
+
+	snprintf(combuf, sizeof(combuf), "-c%d", level);
+	command_add_arg(cmd, combuf);
+}
+
+static void
+command_decompress_init(struct command *cmd, const char *name, const char *desc)
+{
+	command_init(cmd, name, desc);
+	command_add_arg(cmd, name);
+	command_add_arg(cmd, "-dc");
 }
 #endif
 
@@ -275,17 +284,27 @@ static void
 decompress_gzip(struct compress_params *params, int fd_in, int fd_out,
                 const char *desc)
 {
-	fd_fd_filter(fd_in, fd_out, desc, env_gzip, GZIP, "-dc", NULL);
+	struct command cmd;
+
+	command_decompress_init(&cmd, GZIP, desc);
+
+	fd_fd_filter(&cmd, fd_in, fd_out, env_gzip);
+
+	command_destroy(&cmd);
 }
 
 static void
 compress_gzip(struct compress_params *params, int fd_in, int fd_out,
               const char *desc)
 {
-	char combuf[6];
+	struct command cmd;
 
-	snprintf(combuf, sizeof(combuf), "-c%d", params->level);
-	fd_fd_filter(fd_in, fd_out, desc, env_gzip, GZIP, "-n", combuf, NULL);
+	command_compress_init(&cmd, GZIP, desc, params->level);
+	command_add_arg(&cmd, "-n");
+
+	fd_fd_filter(&cmd, fd_in, fd_out, env_gzip);
+
+	command_destroy(&cmd);
 }
 #endif
 
@@ -417,17 +436,26 @@ static void
 decompress_bzip2(struct compress_params *params, int fd_in, int fd_out,
                  const char *desc)
 {
-	fd_fd_filter(fd_in, fd_out, desc, env_bzip2, BZIP2, "-dc", NULL);
+	struct command cmd;
+
+	command_decompress_init(&cmd, BZIP2, desc);
+
+	fd_fd_filter(&cmd, fd_in, fd_out, env_bzip2);
+
+	command_destroy(&cmd);
 }
 
 static void
 compress_bzip2(struct compress_params *params, int fd_in, int fd_out,
                const char *desc)
 {
-	char combuf[6];
+	struct command cmd;
 
-	snprintf(combuf, sizeof(combuf), "-c%d", params->level);
-	fd_fd_filter(fd_in, fd_out, desc, env_bzip2, BZIP2, combuf, NULL);
+	command_compress_init(&cmd, BZIP2, desc, params->level);
+
+	fd_fd_filter(&cmd, fd_in, fd_out, env_bzip2);
+
+	command_destroy(&cmd);
 }
 #endif
 
@@ -568,83 +596,6 @@ filter_lzma_error(struct io_lzma *io, lzma_ret ret)
 }
 
 #ifdef HAVE_LZMA_MT_ENCODER
-# ifdef __linux__
-/*
- * An estimate of how much memory is available. Swap will not be used, the
- * page cache may be purged, not everything will be reclaimed that might be
- * reclaimed, watermarks are considered.
- */
-static const char str_MemAvailable[] = "MemAvailable";
-static const size_t len_MemAvailable = sizeof(str_MemAvailable) - 1;
-
-static int
-get_avail_mem(uint64_t *val)
-{
-	char buf[4096];
-	char *str;
-	ssize_t bytes;
-	int fd;
-
-	*val = 0;
-
-	fd = open("/proc/meminfo", O_RDONLY);
-	if (fd < 0)
-		return -1;
-
-	bytes = fd_read(fd, buf, sizeof(buf));
-	close(fd);
-
-	if (bytes <= 0)
-		return -1;
-
-	buf[bytes] = '\0';
-
-	str = buf;
-	while (1) {
-		char *end;
-
-		end = strchr(str, ':');
-		if (end == 0)
-			break;
-
-		if ((end - str) == len_MemAvailable &&
-		    strncmp(str, str_MemAvailable, len_MemAvailable) == 0) {
-			intmax_t num;
-
-			str = end + 1;
-			errno = 0;
-			num = strtoimax(str, &end, 10);
-			if (num <= 0)
-				return -1;
-			if ((num == INTMAX_MAX) && errno == ERANGE)
-				return -1;
-			/* It should end with ' kB\n'. */
-			if (*end != ' ' || *(end + 1) != 'k' ||
-			    *(end + 2) != 'B')
-				return -1;
-
-			/* This should not overflow, but just in case. */
-			if (num < (INTMAX_MAX / 1024))
-				num *= 1024;
-			*val = num;
-			return 0;
-		}
-
-		end = strchr(end + 1, '\n');
-		if (end == 0)
-			break;
-		str = end + 1;
-	}
-	return -1;
-}
-# else
-static int
-get_avail_mem(uint64_t *val)
-{
-	return -1;
-}
-# endif
-
 static uint64_t
 filter_xz_get_memlimit(void)
 {
@@ -653,7 +604,7 @@ filter_xz_get_memlimit(void)
 	/* Ask the kernel what is currently available for us. If this fails
 	 * initialize the memory limit to half the physical RAM, or to 128 MiB
 	 * if we cannot infer the number. */
-	if (get_avail_mem(&mt_memlimit) < 0) {
+	if (meminfo_get_available(&mt_memlimit) < 0) {
 		mt_memlimit = lzma_physmem() / 2;
 		if (mt_memlimit == 0)
 			mt_memlimit = 128 * 1024 * 1024;
@@ -667,14 +618,17 @@ filter_xz_get_memlimit(void)
 }
 
 static uint32_t
-filter_xz_get_cputhreads(void)
+filter_xz_get_cputhreads(struct compress_params *params)
 {
-	uint32_t threads_max;
+	long threads_max;
 
 	threads_max = lzma_cputhreads();
 	if (threads_max == 0)
 		threads_max = 1;
 
+	if (params->threads_max >= 0)
+		return clamp(params->threads_max, 1, threads_max);
+
 	return threads_max;
 }
 #endif
@@ -718,7 +672,7 @@ filter_xz_init(struct io_lzma *io, lzma_stream *s)
 #ifdef HAVE_LZMA_MT_ENCODER
 	mt_options.preset = preset;
 	mt_memlimit = filter_xz_get_memlimit();
-	mt_options.threads = filter_xz_get_cputhreads();
+	mt_options.threads = filter_xz_get_cputhreads(io->params);
 
 	/* Guess whether we have enough RAM to use the multi-threaded encoder,
 	 * and decrease them up to single-threaded to reduce memory usage. */
@@ -793,23 +747,36 @@ static void
 decompress_xz(struct compress_params *params, int fd_in, int fd_out,
               const char *desc)
 {
-	fd_fd_filter(fd_in, fd_out, desc, env_xz, XZ, "-dc", NULL);
+	struct command cmd;
+
+	command_decompress_init(&cmd, XZ, desc);
+
+	fd_fd_filter(&cmd, fd_in, fd_out, env_xz);
+
+	command_destroy(&cmd);
 }
 
 static void
 compress_xz(struct compress_params *params, int fd_in, int fd_out,
             const char *desc)
 {
-	char combuf[6];
-	const char *strategy;
+	struct command cmd;
+	char *threads_opt = NULL;
+
+	command_compress_init(&cmd, XZ, desc, params->level);
 
 	if (params->strategy == COMPRESSOR_STRATEGY_EXTREME)
-		strategy = "-e";
-	else
-		strategy = NULL;
+		command_add_arg(&cmd, "-e");
 
-	snprintf(combuf, sizeof(combuf), "-c%d", params->level);
-	fd_fd_filter(fd_in, fd_out, desc, env_xz, XZ, combuf, strategy, NULL);
+	if (params->threads_max > 0) {
+		threads_opt = str_fmt("-T%d", params->threads_max);
+		command_add_arg(&cmd, threads_opt);
+	}
+
+	fd_fd_filter(&cmd, fd_in, fd_out, env_xz);
+
+	command_destroy(&cmd);
+	free(threads_opt);
 }
 #endif
 
@@ -894,17 +861,28 @@ static void
 decompress_lzma(struct compress_params *params, int fd_in, int fd_out,
                 const char *desc)
 {
-	fd_fd_filter(fd_in, fd_out, desc, env_xz, XZ, "-dc", "--format=lzma", NULL);
+	struct command cmd;
+
+	command_decompress_init(&cmd, XZ, desc);
+	command_add_arg(&cmd, "--format=lzma");
+
+	fd_fd_filter(&cmd, fd_in, fd_out, env_xz);
+
+	command_destroy(&cmd);
 }
 
 static void
 compress_lzma(struct compress_params *params, int fd_in, int fd_out,
               const char *desc)
 {
-	char combuf[6];
+	struct command cmd;
 
-	snprintf(combuf, sizeof(combuf), "-c%d", params->level);
-	fd_fd_filter(fd_in, fd_out, desc, env_xz, XZ, combuf, "--format=lzma", NULL);
+	command_compress_init(&cmd, XZ, desc, params->level);
+	command_add_arg(&cmd, "--format=lzma");
+
+	fd_fd_filter(&cmd, fd_in, fd_out, env_xz);
+
+	command_destroy(&cmd);
 }
 #endif
 
