@@ -36,7 +36,6 @@ use Dpkg::Source::Patch;
 use Dpkg::Exit qw(push_exit_handler pop_exit_handler);
 use Dpkg::Source::Functions qw(erasedir);
 use Dpkg::Source::Package::V3::Native;
-use Dpkg::OpenPGP;
 
 use parent qw(Dpkg::Source::Package);
 
@@ -63,6 +62,10 @@ sub init_options {
     $self->{options}{ignore_bad_version} //= 0;
     $self->{options}{abort_on_upstream_changes} //= 0;
 
+    # Set default validation checks.
+    $self->{options}{require_valid_signature} //= 0;
+    $self->{options}{require_strong_checksums} //= 0;
+
     # V1.0 only supports gzip compression.
     $self->{options}{compression} //= 'gzip';
     $self->{options}{comp_level} //= compression_get_property('gzip', 'default_level');
@@ -162,8 +165,6 @@ sub do_extract {
 	         $sourcestyle);
     }
 
-    my $dscdir = $self->{basedir};
-
     my $basename = $self->get_basename();
     my $basenamerev = $self->get_basename(1);
 
@@ -209,7 +210,9 @@ sub do_extract {
         }
 
         info(g_('unpacking %s'), $tarfile);
-        my $tar = Dpkg::Source::Archive->new(filename => "$dscdir$tarfile");
+        my $tar = Dpkg::Source::Archive->new(
+            filename => File::Spec->catfile($self->{basedir}, $tarfile),
+        );
         $tar->extract($expectprefix);
 
         if ($sourcestyle =~ /u/) {
@@ -234,7 +237,7 @@ sub do_extract {
     }
 
     if ($difffile and not $self->{options}{skip_debianization}) {
-        my $patch = "$dscdir$difffile";
+        my $patch = File::Spec->catfile($self->{basedir}, $difffile);
 	info(g_('applying %s'), $difffile);
 	my $patch_obj = Dpkg::Source::Patch->new(filename => $patch);
 	my $analysis = $patch_obj->apply($newdirectory, force_timestamp => 1);
@@ -267,7 +270,7 @@ sub do_build {
                     'argument (with v1.0 source package)'));
     }
 
-    $sourcestyle =~ y/X/A/;
+    $sourcestyle =~ y/X/a/;
     unless ($sourcestyle =~ m/[akpursnAKPUR]/) {
         usageerr(g_('source handling style -s%s not allowed with -b'),
                  $sourcestyle);
@@ -347,6 +350,16 @@ sub do_build {
 	}
     }
 
+    my $v = Dpkg::Version->new($self->{fields}->{'Version'});
+    if ($sourcestyle =~ m/[kpursKPUR]/) {
+        error(g_('non-native package version does not contain a revision'))
+            if $v->is_native();
+    } else {
+        # TODO: This will become fatal in the near future.
+        warning(g_('native package version may not have a revision'))
+            unless $v->is_native();
+    }
+
     my ($dirname, $dirbase) = fileparse($dir);
     if ($dirname ne $basedirname) {
 	warning(g_("source directory '%s' is not <sourcepackage>" .
@@ -409,13 +422,22 @@ sub do_build {
 	     $sourcepackage, $tarname);
     }
 
-    $self->add_file($tarname) if $tarname;
-    if ($tarname and -e "$tarname.sig" and not -e "$tarname.asc") {
-        openpgp_sig_to_asc("$tarname.sig", "$tarname.asc");
+    if ($tarname) {
+        $self->add_file($tarname);
+        if (-e "$tarname.sig" and not -e "$tarname.asc") {
+            $self->armor_original_tarball_signature("$tarname.sig", "$tarname.asc");
+        }
     }
     if ($tarsign and -e $tarsign) {
+        $self->check_original_tarball_signature($dir, $tarsign);
+
         info(g_('building %s using existing %s'), $sourcepackage, $tarsign);
         $self->add_file($tarsign);
+    } else {
+        my $key = $self->get_upstream_signing_key($dir);
+        if (-e $key) {
+            warning(g_('upstream signing key but no upstream tarball signature'));
+        }
     }
 
     if ($sourcestyle =~ m/[kpKP]/) {
@@ -425,9 +447,7 @@ sub do_build {
                          'giving up; use -sA, -sK or -sP to override'),
                       $origdir);
             }
-            push_exit_handler(sub { erasedir($origdir) });
             erasedir($origdir);
-            pop_exit_handler();
         } elsif ($! != ENOENT) {
             syserr(g_("unable to check for existence of orig directory '%s'"),
                     $origdir);
