@@ -56,7 +56,6 @@ use Dpkg::Vendor qw(run_vendor_hook);
 textdomain('dpkg-dev');
 
 my $controlfile;
-my $changelogfile;
 my $changelogformat;
 
 my $build_format;
@@ -164,14 +163,16 @@ while (@options) {
     } elsif (m/^-c(.*)$/) {
         $controlfile = $1;
     } elsif (m/^-l(.*)$/) {
-        $changelogfile = $1;
+        $options{changelog_file} = $1;
     } elsif (m/^-F([0-9a-z]+)$/) {
         $changelogformat = $1;
     } elsif (m/^-D([^\=:]+)[=:](.*)$/s) {
         $override{$1} = $2;
     } elsif (m/^-U([^\=:]+)$/) {
         $remove{$1} = 1;
-    } elsif (m/^-(?:i|-diff-ignore(?:$|=))(.*)$/) {
+    } elsif (m/^--diff-ignore$/) {
+        $options{diff_ignore_regex} = $diff_ignore_regex;
+    } elsif (m/^-(?:i|-diff-ignore=)(.*)$/) {
         $options{diff_ignore_regex} = $1 ? $1 : $diff_ignore_regex;
     } elsif (m/^--extend-diff-ignore=(.+)$/) {
 	$diff_ignore_regex .= "|$1";
@@ -228,10 +229,10 @@ if ($options{opmode} =~ /^(build|print-format|(before|after)-build|commit)$/) {
 
     $options{ARGV} = \@ARGV;
 
-    $changelogfile ||= "$dir/debian/changelog";
+    $options{changelog_file} ||= "$dir/debian/changelog";
     $controlfile ||= "$dir/debian/control";
 
-    my %ch_options = (file => $changelogfile);
+    my %ch_options = (file => $options{changelog_file});
     $ch_options{changelogformat} = $changelogformat if $changelogformat;
     my $changelog = changelog_parse(%ch_options);
     my $control = Dpkg::Control::Info->new($controlfile);
@@ -317,7 +318,9 @@ if ($options{opmode} =~ /^(build|print-format|(before|after)-build|commit)$/) {
             $profile =~ s/\s+/,/g;
             $pkg_summary .= " profile=$profile";
         }
-
+        if (defined $pkg->{'Protected'} and $pkg->{'Protected'} eq 'yes') {
+            $pkg_summary .= ' protected=yes';
+        }
         if (defined $pkg->{'Essential'} and $pkg->{'Essential'} eq 'yes') {
             $pkg_summary .= ' essential=yes';
         }
@@ -525,6 +528,11 @@ sub set_testsuite_triggers_field
     return if $fields->{'Testsuite-Triggers'};
 
     foreach my $test ($tests->get()) {
+        if (not exists $test->{Tests} and not exists $test->{'Test-Command'}) {
+            error(g_('test control %s is missing %s or %s field'),
+                  'debian/tests/control', 'Tests', 'Test-Command');
+        }
+
         next unless $test->{Depends};
 
         my $deps = deps_parse($test->{Depends}, use_arch => 0, tests_dep => 1);
@@ -550,12 +558,12 @@ sub setopmode {
 
 sub print_option {
     my $opt = shift;
-    my $help;
 
+    my $help = gettext($opt->{help});
     if (length $opt->{name} > 25) {
-        $help .= sprintf "  %-25s\n%s%s.\n", $opt->{name}, ' ' x 27, $opt->{help};
+        return sprintf "  %-25s\n%s%s.\n", $opt->{name}, ' ' x 27, $help;
     } else {
-        $help .= sprintf "  %-25s%s.\n", $opt->{name}, $opt->{help};
+        return sprintf "  %-25s%s.\n", $opt->{name}, $help;
     }
 }
 
